// subor ModelSwarm.java

import swarm.Globals;
import swarm.Selector;
import swarm.defobj.Zone;
import swarm.defobj.SymbolImpl;

import swarm.defobj.FArguments;
import swarm.defobj.FArgumentsImpl;
import swarm.defobj.FCall;
import swarm.defobj.FCallImpl;

import swarm.activity.Activity;
import swarm.activity.ActionGroup;
import swarm.activity.ActionGroupImpl;
import swarm.activity.Schedule;
import swarm.activity.ScheduleImpl;
import swarm.activity.FActionForEach;

import swarm.objectbase.Swarm;
import swarm.objectbase.SwarmImpl;
import swarm.objectbase.VarProbe;
import swarm.objectbase.MessageProbe;
import swarm.objectbase.EmptyProbeMapImpl;

import java.util.LinkedList;
import java.util.List;

import swarm.space.Grid2d;
import swarm.space.Grid2dImpl;

public class ModelSwarm extends SwarmImpl
{
	public boolean debug = true;
	
	public int worldXSize, worldYSize;
	public double seedProb;
	public double bugDensity;
	
	public FoodSpace food;
	public Grid2d world;
	
	public List bugList;
	public ActionGroup modelActions;
	public Schedule modelSchedule;
	
	public boolean randomizeBugUpdateOrder;
	
	public FActionForEach actionForEach;    // ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
	public ModelSwarm(Zone aZone)    // createBegin()
	{
		super(aZone);
		
		worldXSize = 80;
		worldYSize = 80;
		seedProb = 0.5;
		bugDensity = 0.05;
		randomizeBugUpdateOrder = false;
		
		class BugModelProbeMap extends EmptyProbeMapImpl {
      private VarProbe probeVariable (String name) {
        return
          Globals.env.probeLibrary.getProbeForVariable$inClass
          (name, ModelSwarm.this.getClass ());
      }
      private MessageProbe probeMessage (String name) {
        return
          Globals.env.probeLibrary.getProbeForMessage$inClass
          (name, ModelSwarm.this.getClass ());
      }
      private void addVar (String name) {
        addProbe (probeVariable (name));
      }
      private void addMessage (String name) {
        addProbe (probeMessage (name));
      }
      public BugModelProbeMap (Zone _aZone, Class aClass) {
        super (_aZone, aClass);
        addVar ("bugDensity");
        addVar ("seedProb");
        addVar ("worldXSize");
        addVar ("worldYSize");
        addMessage ("addBug:");
      }
    }
    
    // Now, install our custom probeMap class directly into the
    // probeLibrary
    Globals.env.probeLibrary.setProbeMap$For
      (new BugModelProbeMap (aZone, getClass ()), getClass ());
		
	}
	
	public Object addBug(Bug bug)
	{
		bugList.add(bug);
		return this;
	}
	
	public List getBugList()
	{
		return bugList;
	}
	
	public Grid2d getWorld()
	{
		return world;
	}
	
	public FoodSpace getFood()
	{
		return food;
	}
	
	public void syncUpdateOrder()
	{
		if(actionForEach != null)
		{
			actionForEach.setDefaultOrder(randomizeBugUpdateOrder ? Globals.env.Randomized : Globals.env.Sequential);
		}
	}
	
	public Object buildObjects()
	{
		Bug aBug;
		int x, y;
		
		super.buildObjects();
		
		food = new FoodSpace(getZone(), worldXSize, worldYSize);
		if(debug)
		{
			if(food == null)
				System.out.println("ModelSwarm.java: createObject(): food referens to null");
			else
				System.out.println("ModelSwarm.java: createObject(): food OK");
			System.out.println("ModelSwarm.java: food.seedFoodWithProb()");	
		}
		food.seedFoodWithProb(seedProb);
		
		if(debug)
			System.out.println("ModelSwarm.java: createObject(): creating world");
		world = new Grid2dImpl(getZone(), worldXSize, worldYSize);
		if(debug)
			if(world == null)
				System.out.println("ModelSwarm.java: createObject(): world referens to null");
			else
				System.out.println("ModelSwarm.java: createObject(): world OK");
		
		world.fillWithObject(null);
		
		bugList = new LinkedList();
		
		for(y = 0; y < worldYSize; y++)
			for(x = 0; x < worldXSize; x++)
				if(Math.random() < bugDensity)
				{
					aBug = new Bug(getZone());
					aBug.setWorld$Food(world, food);
					aBug.createEnd();
					aBug.setX$Y(x, y);
					
					bugList.add(aBug);
				}
		
		return this;
	}
	
	
	public Object buildActions()
	{
		super.buildActions();
		
		modelActions = new ActionGroupImpl(getZone());
		
		if(debug)
		{
			if(bugList == null)
				System.out.println("ModelSwarm: buildActions(): bugList == null");
			if(modelActions == null)
				System.out.println("ModelSwarm: buildActions(): modelActions == null");
		}
		
		/* tento blok stale vyhadzoval vynimku...
		try
		{
			modelActions.createFActionForEach$message(bugList, new Selector(bugList.getClass(), "step", false)); 
		}
		catch(Exception e)
		{
			System.out.println("ModelSwarm: buildActions(): Selector exception ");
		} 
		*/
		
		try
		{
			if(debug && (bugList == null))
				System.out.println("ModelSwarm: buildActions: bugList == null");
			Bug proto = (Bug) bugList.get(0);
			if(debug && (proto == null))
				System.out.println("ModelSwarm: buildActions: proto == null");
			Selector sel = new Selector(proto.getClass(), "step", false);
			if(debug && (sel == null))
				System.out.println("ModelSwarm: buildActions: sel == null");
			actionForEach = modelActions.createFActionForEachHomogeneous$call(bugList, new FCallImpl(this, proto, sel, new FArgumentsImpl(this, sel, true)));
			if(debug && (actionForEach == null))
				System.out.println("ModelSwarm: buildActions: actionForEach == null");
		} 
		catch(Exception e)
		{
			System.out.println("ModelSwarm: buildActions(): New Selector Exception");
		}
		
		// ++++++++++++++++++++++++++++++++++++++++++++++++++
		syncUpdateOrder();
		// ++++++++++++++++++++++++++++++++++++++++++++++++++
		
		if(debug)
			System.out.println("ModelSwarm: buildActions(): creating modelSchedule");		
		modelSchedule = new ScheduleImpl(getZone(), 1);
		if(debug && (modelSchedule == null))
				System.out.println("ModelSwarm: buildActions(): modelSchedule == null");
		modelSchedule.at$createAction(0, modelActions);
		if(debug)
				System.out.println("ModelSwarm: buildActions(): finished");
		return this;
	}
	
	
	public Activity activateIn(Swarm swarmContext)
	{
		if(debug)
				System.out.println("ModelSwarm: activateIn(): started");
		super.activateIn(swarmContext);
		if(debug && (modelSchedule == null))
				System.out.println("ModelSwarm: activateIn(): modelSchedule == null");
		modelSchedule.activateIn(this);
		if(debug)
				System.out.println("ModelSwarm: activatIn(): finishing");		
		return this.getActivity();
	}

}

