// subor ObserverSwarm.java

import swarm.Globals;
import swarm.Selector;
import swarm.defobj.Zone;

import swarm.activity.Activity;
import swarm.activity.ActionGroup;
import swarm.activity.ActionGroupImpl;
import swarm.activity.Schedule;
import swarm.activity.ScheduleImpl;

import swarm.objectbase.Swarm;
import swarm.objectbase.VarProbe;
import swarm.objectbase.MessageProbe;
import swarm.objectbase.EmptyProbeMapImpl;

import swarm.gui.Colormap;
import swarm.gui.ColormapImpl;
import swarm.gui.ZoomRaster;
import swarm.gui.ZoomRasterImpl;

import swarm.analysis.EZGraph;
import swarm.analysis.EZGraphImpl;

import swarm.simtoolsgui.GUISwarm;
import swarm.simtoolsgui.GUISwarmImpl;

import swarm.space.Value2dDisplay;
import swarm.space.Value2dDisplayImpl;
import swarm.space.Object2dDisplay;
import swarm.space.Object2dDisplayImpl;

import java.util.List;

public class ObserverSwarm extends GUISwarmImpl
{
	public boolean debug = false;
	
	public int displayFrequency;
	
	public ActionGroup displayActions;				
  public Schedule displaySchedule;
	
	public ModelSwarm modelSwarm;
	
	public Colormap colorMap;
	public ZoomRaster worldRaster;
	
	public Value2dDisplay foodDisplay;
	public Object2dDisplay bugDisplay;
	
	public ObserverSwarm(Zone aZone)
	{
		super(aZone);
		displayFrequency = 1;
		
		class BugObserverProbeMap extends EmptyProbeMapImpl {
      private VarProbe probeVariable (String name) {
        return
          Globals.env.probeLibrary.getProbeForVariable$inClass
          (name, ObserverSwarm.this.getClass ());
      }
      private MessageProbe probeMessage (String name) {
        return
          Globals.env.probeLibrary.getProbeForMessage$inClass
          (name, ObserverSwarm.this.getClass ());
      }
      private void addVar (String name) {
        addProbe (probeVariable (name));
      }
      private void addMessage (String name) {
        addProbe (probeMessage (name));
      }
      public BugObserverProbeMap (Zone _aZone, Class aClass) {
        super (_aZone, aClass);
        addVar ("displayFrequency");
        // addMessage ("graphBug:");
      }
    } 
        
    // Install our custom probeMap class directly into the
    // probeLibrary
    Globals.env.probeLibrary.setProbeMap$For
      (new BugObserverProbeMap (aZone, getClass ()), getClass ());
		
	} // koniec konstruktora
	
	// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	public Object _worldRasterDeath_(Object caller)
	{
		worldRaster.drop();
		worldRaster = null;
		return this;
	}
	// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
	public Object buildObjects()
	{
		super.buildObjects();
		
		if(debug)
			System.out.println("ObserverSwarm: super.buildObjects() finished");
		modelSwarm = new ModelSwarm(getZone());
		Globals.env.createArchivedProbeDisplay(modelSwarm, "modelSwarm");
    Globals.env.createArchivedProbeDisplay(this, "observerSwarm");	
		getControlPanel().setStateStopped();
		if(debug)
			System.out.println("ObserverSwarm: modelSwarm.buildObjects()");
		modelSwarm.buildObjects();
		
		if(debug)
			System.out.println("ObserverSwarm: Setting colorMap");
		colorMap = new ColormapImpl(getZone());
		colorMap.setColor$ToName((byte)0, "black");
		colorMap.setColor$ToName((byte)1, "red");
		colorMap.setColor$ToName((byte)2, "green");
		
		// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		List bugList = modelSwarm.getBugList();
		/*
		for(int i = 0; i < bugList.size(); i++)
		{
			Bug bug = (Bug)bugList.get(i);
			bug.setBugColor((byte)2);
		} */
		// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		
		if(debug)
			System.out.println("ObserverSwarm: Setting worldRaster: running constructor");
		worldRaster = new ZoomRasterImpl(getZone(), "worldRaster");
		
		// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		try
		{
			worldRaster.enableDestroyNotification$notificationMethod(this, new Selector(getClass(), "_worldRasterDeath_", false));
		}
		catch(Exception e)
		{
			System.out.println("ObserverSwarm:Setting vorldRaster: Selector exception");
		}
		// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		
		if(debug)
			System.out.println("ObserverSwarm: Setting worldRaster: setColormap()");
		worldRaster.setColormap(colorMap);
		
		if(debug)
			System.out.println("ObserverSwarm: Setting worldRaster: setZoomFactor");
		worldRaster.setZoomFactor(4);
		
		if(debug)
			System.out.println("ObserverSwarm: Setting worldRaster: setWidth$Hight()");
		// worldRaster.setWidth$Height(80, 80);
		worldRaster.setWidth$Height(modelSwarm.getWorld().getSizeX(), modelSwarm.getWorld().getSizeX());
		
		if(debug)
			System.out.println("ObserverSwarm: Setting worldRaster: setWindowTitle()");
		worldRaster.setWindowTitle("Food World");
		
		if(debug)
			System.out.println("ObserverSwarm: Setting worldRaster: pack()");
		worldRaster.pack();
		
		if(debug)
		{
			System.out.println("ObserverSwarm: Setting foodDisplay");
			if(worldRaster == null)
				System.out.println("               worldRaster == null");
			if(colorMap == null)
				System.out.println("               colorMap == null");
			if(modelSwarm.getFood() == null)
				System.out.println("               modelSwarm.getFood() returns null");
		}
						
		foodDisplay = new Value2dDisplayImpl(getZone(), worldRaster, colorMap, modelSwarm.getFood());
		
		// +++++++++++++++++++++++++++++++++++++++
		// +++++++++++++++++++++++++++++++++++++++
		foodDisplay.setDisplayMappingM$C(1, 0);
		// +++++++++++++++++++++++++++++++++++++++
		// +++++++++++++++++++++++++++++++++++++++
		
		if(debug)
			System.out.println("ObserverSwarm: Setting bugDisplay");
		try 
		{
    	bugDisplay = new Object2dDisplayImpl(getZone(), worldRaster, modelSwarm.getWorld(),
         new Selector(Class.forName("Bug"), "drawSelfOn", false)); 
		} catch (Exception e) 
		{
			System.out.println("ObserverSwarm: Setting bugDisplay: Selector Exception");
    }
		
		if(debug)
			System.out.println("ObserverSwarm: Setting bugDisplay: setObjectCollection()");
		bugDisplay.setObjectCollection(modelSwarm.getBugList());
		// bugDisplay.setDisplayMessage("drawSelfOn:");
		
		// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		try
		{
			worldRaster.setButton$Client$Message(3, bugDisplay, new Selector(bugDisplay.getClass(), "makeProbeAtX$Y", true));
		} catch(Exception e)
		{
			System.out.println("ObserverSwarm: makeProbeAtX$Y Exception");
		}
		// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	
		return this;
	}
	
	public Object _update_()
	{
		if(worldRaster != null)
		{
			foodDisplay.display();
			bugDisplay.display();
			worldRaster.drawSelf();
		}
		return this;
	}
	
	public Object buildActions()
	{
		super.buildActions();
		//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		if(debug)
			System.out.println("ObserverSwarm: buildActions(): modelSwarm.buildActions()");
		modelSwarm.buildActions();
		// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		if(debug)
			System.out.println("ObserverSwarm: buildActions(): create displayActions");
		displayActions = new ActionGroupImpl(getZone());
		// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		if(debug)
			System.out.println("ObserverSwarm: buildActions(): try block");
		// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		if(debug && (displayActions == null))
			System.out.println("ObserverSwarm: buildActions(): displayActions == null");
		try
		{
			// +++++++++++++++++++++++++++++++++++++++++++++++++++
			displayActions.createActionTo$message(this, new Selector(getClass(), "_update_", false));
			// displayActions.createActionTo$message(Globals.env.probeDisplayManager.getClass(), new Selector(Globals.env.probeDisplayManager.getClass(), "update", true));
			// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
			displayActions.createActionTo$message(foodDisplay, new Selector(foodDisplay.getClass(), "display", false));
			displayActions.createActionTo$message(bugDisplay, new Selector(bugDisplay.getClass(), "display", false));
			displayActions.createActionTo$message(worldRaster, new Selector(worldRaster.getClass(), "drawSelf", false));
			displayActions.createActionTo$message(getActionCache(), new Selector(getActionCache().getClass(), "doTkEvents", true));
		}catch(Exception e)
		{
			System.out.println("ObserverSwarm: buildActions(): Selector exception");
		}
		// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		if(debug)
			System.out.println("ObserverSwarm: buildActions(): create displaySchedule");
		displaySchedule = new ScheduleImpl(getZone(), displayFrequency);
		// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		if(debug)
			System.out.println("ObserverSwarm: buildActions(): setRepeatInterval()");
		// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		if(debug)
			System.out.println("ObserverSwarm: buildActions(): at$crateAction()");
		displaySchedule.at$createAction(0, displayActions);
		
		return this;
	}
	
	public Activity activateIn(Swarm swarmContext)
	{
		if(debug)
		{
			System.out.println("ObserverSwarm: activateIn(): super.activateIn()");
		}
		super.activateIn(swarmContext);
		// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		if(debug)
		{
			System.out.println("ObserverSwarm: activateIn(): modelSwarm.activateIn()");
			if(modelSwarm == null)
				System.out.println("ObserverSwarm: activateIn(): modelSwarm refer to null");
		}
		modelSwarm.activateIn(this);
		// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		if(debug)
		{
			System.out.println("ObserverSwarm: activateIn(): displayShedule.activateIn()");
			if(displaySchedule == null)
				System.out.println("ObserverSwarm: activateIn(): displaySchedule refer to null");
		}
		displaySchedule.activateIn(this);
		// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		if(debug)
		{
			System.out.println("ObserverSwarm: activateIn(): return getActivity()");
			if(getActivity() == null)
				System.out.println("ObserverSwarm: activateIn(): getActivity() returns null");
		}
		return getActivity();
	}
	
	public void drop()
	{
		if(worldRaster != null)
		{
			worldRaster.disableDestroyNotification();
		}
		super.drop();
	}

}

